/*
 * PSGLGraphics.cpp
 *
 *  Created on: Oct 26, 2010
 *      Author: halsafar
 */

#include "PSGLGraphics.h"

#include <stdio.h>
#include <string.h>
#include <sys/sys_time.h>

#include "../logger/Logger.h"

PSGLGraphics::PSGLGraphics()
{
	LOG("~PSGLGraphics - constructor");
	psgl_device = NULL;
	psgl_context = NULL;
	gl_width = 0;
	gl_height = 0;
}

PSGLGraphics::~PSGLGraphics()
{
	LOG("~PSGLGraphics - destructor");

}

PSGLdeviceParameters PSGLGraphics::InitCommon(uint32_t resolutionId, uint16_t pal60Hz)
{
	LOG("PSGLGraphics::InitCommon(uint32_t resolutionId, uint16_t pal60Hz)\n");
	PSGLinitOptions options =
	{
		enable: PSGL_INIT_MAX_SPUS | PSGL_INIT_INITIALIZE_SPUS | PSGL_INIT_HOST_MEMORY_SIZE,
		maxSPUs: 1,
		initializeSPUs: false,
		persistentMemorySize: 0,
		transientMemorySize: 0,
		errorConsole: 0,
		fifoSize: 0,
		hostMemorySize: 8 * 1024 * 1024
	};

	LOG("PSGLGraphics::InitCommon()->psglInit(&options)\n");
	psglInit(&options);

	PSGLdeviceParameters params;
	params.enable = PSGL_DEVICE_PARAMETERS_COLOR_FORMAT | PSGL_DEVICE_PARAMETERS_DEPTH_FORMAT | PSGL_DEVICE_PARAMETERS_MULTISAMPLING_MODE | PSGL_DEVICE_PARAMETERS_RESC_PAL_TEMPORAL_MODE;
	params.colorFormat = GL_ARGB_SCE;
	params.depthFormat = GL_NONE;
	params.multisamplingMode = GL_MULTISAMPLING_NONE_SCE;

	if(pal60Hz)
	{
		LOG("PSGLGraphics::InitCommon()->PAL60\n");
		params.enable |= PSGL_DEVICE_PARAMETERS_RESC_PAL_TEMPORAL_MODE;
		params.rescPalTemporalMode = RESC_PAL_TEMPORAL_MODE_60_INTERPOLATE;
	}

	if(resolutionId != NULL)
	{
		LOG("PSGLGraphics::InitCommon()->resolutionId != NULL\n");
		//Resolution setting
		CellVideoOutResolution resolution;
		cellVideoOutGetResolution(resolutionId, &resolution);
		
		params.enable |= PSGL_DEVICE_PARAMETERS_WIDTH_HEIGHT;
		params.width = resolution.width;
		params.height = resolution.height;
	}

	params.enable |= PSGL_DEVICE_PARAMETERS_RESC_RATIO_MODE;
	params.rescRatioMode = RESC_RATIO_MODE_FULLSCREEN;
	LOG("PSGLGraphics::InitCommon()->return params\n");
	return params;
}

void PSGLGraphics::Init()
{
	LOG("PSGLGraphics::Init()\n");
	PSGLdeviceParameters myParams;
	myParams	= InitCommon(NULL, 0);

	LOG("PSGLGraphics::Init()->psglCreateDeviceExtended(&myParams)\n");
	psgl_device = psglCreateDeviceExtended(&myParams);


	LOG("PSGLGraphics::Init()->psglGetDeviceDimensions(psgl_device, &gl_width, &gl_height)\n");
	psglGetDeviceDimensions(psgl_device, &gl_width, &gl_height); // Get the dimensions of the screen in question, and do stuff with it :)

	LOG("PSGLGraphics::Init()->psgl_context = psglCreateContext()\n");
	psgl_context = psglCreateContext(); // Create a context and bind it to the current display.
	

	LOG("PSGLGraphics::Init()->psglMakeCurrent(psgl_context, psgl_device)\n");
	psglMakeCurrent(psgl_context, psgl_device);

	LOG("PSGLGraphics::Init()->psglResetCurrentContext()\n");
	psglResetCurrentContext();
}

void PSGLGraphics::Init(uint32_t resolutionId, uint16_t pal60Hz)
{
	LOG("PSGLGraphics::Init(uint32_t resolutionId, uint16_t pal60Hz)\n");
	PSGLdeviceParameters myParams;
	myParams	= InitCommon(resolutionId, pal60Hz);

	LOG("PSGLGraphics::Init(uint32_t resolutionId, uint16_t pal60Hz)->psglCreateDeviceExtended(&myParams)\n");
	psgl_device = psglCreateDeviceExtended(&myParams);

	LOG("PSGLGraphics::Init(uint32_t resolutionId, uint16_t pal60Hz)->psglGetDeviceDimensions(psgl_device, &gl_width, &gl_height)\n");
	psglGetDeviceDimensions(psgl_device, &gl_width, &gl_height); // Get the dimensions of the screen in question, and do stuff with it :)

	LOG("PSGLGraphics::Init(uint32_t resolutionId, uint16_t pal60Hz)->psgl_context = psglCreateContext()\n");
	psgl_context = psglCreateContext(); // Create a context and bind it to the current display.

	LOG("PSGLGraphics::Init(uint32_t resolutionId, uint16_t pal60Hz)->psglMakeCurrent(psgl_context, psgl_device)\n");
	psglMakeCurrent(psgl_context, psgl_device);

	LOG("PSGLGraphics::Init(uint32_t resolutionId, uint16_t pal60Hz)->psglResetCurrentContext()\n");
	psglResetCurrentContext();
}


void PSGLGraphics::Deinit()
{
   LOG("PSGLGraphics::Deinit()\n");
   glFinish();
   this->DeinitDbgFont();
   //free(gl_buffer);
   //free(vertex_buf);

   psglDestroyContext(psgl_context);
   psglDestroyDevice(psgl_device);
#ifdef PS3_SDK_3_41
   //FIXME: It will crash here for 1.92 - termination of the PSGL library - works fine for 3.41
   psglExit();
#else
   //for 1.92
   gl_width = 0;
   gl_height = 0;
   psgl_context = NULL;
   psgl_device = NULL;
#endif
}

void PSGLGraphics::InitDbgFont()
{
   LOG("PSGLGraphics::InitDbgFont()\n");
	CellDbgFontConfig cfg;
	memset(&cfg, 0, sizeof(cfg));
	cfg.bufSize = 512;
	cfg.screenWidth = gl_width;
	cfg.screenHeight = gl_height;
	cellDbgFontInit(&cfg);
}


void PSGLGraphics::DeinitDbgFont()
{
   LOG("PSGLGraphics::DeinitDbgFont()\n");
	cellDbgFontExit();
}


GLfloat PSGLGraphics::GetDeviceAspectRatio()
{
   LOG("PSGLGraphics::GetDeviceAspectRatio()\n");
	return psglGetDeviceAspectRatio(psgl_device);
}


GLint PSGLGraphics::GetResolutionWidth()
{
   LOG("PSGLGraphics::GetResolutionWidth()\n");
	return gl_width;
}


GLint PSGLGraphics::GetResolutionHeight()
{
   LOG("PSGLGraphics::GetResolutionHeight()\n");
	return gl_height;
}




//FIXME: classify this
void dprintf_console(const char* fmt, ...)
{
	glClear(GL_COLOR_BUFFER_BIT);

	va_list ap;

	va_start(ap, fmt);
	cellDbgFontConsoleVprintf(dbg_id, fmt, ap);
	va_end(ap);

	cellDbgFontConsoleEnable(dbg_id);
	cellDbgFontDraw();
	psglSwap();	// added comment by SSNES author - should this be here?
}

void dprintf_noswap(float x, float y, float scale, const char* fmt, ...)
{
   char buffer[512];

   va_list ap;

   va_start(ap, fmt);
   vsnprintf(buffer, 512, fmt, ap);
   cellDbgFontPuts(x, y, scale, 0xffffffff, buffer);
   va_end(ap);

   cellDbgFontDraw();
}

void write_fps(void)
{
   static float last_time = 0.0;
   float new_time = sys_time_get_system_time() / 1000000.0;
   float delta = new_time - last_time;
   last_time = new_time;

   dprintf_noswap(0.1, 0.1, 1.0, "FPS: %.2f\n", 1.0f/delta);
}


void gl_dprintf(float x, float y, float scale, const char* fmt, ...)
{
	glClear(GL_COLOR_BUFFER_BIT);

	char buffer[512];

	va_list ap;

	va_start(ap, fmt);
	vsnprintf(buffer, 512, fmt, ap);
	cellDbgFontPuts(x, y, scale, 0xffffffff, buffer);
	va_end(ap);

	cellDbgFontDraw();
	psglSwap();
}
