#ifndef __NETWORK_POLLER_HPP
#define __NETWORK_POLLER_HPP

#include <vector>
#include <stdint.h>
#include "Socket.hpp"
#include <sys/poll.h>

namespace Network {
   
   class Poller
   {
      public:
         typedef enum PollType {
            IN = POLLIN,
            OUT = POLLOUT
         } PollType;

         struct PollFD
         {
            PollFD(Socket *in, int in_events, int in_revents = 0) : sock(in), events(in_events), revents(in_revents) {}
            Socket *sock;
            int events;
            int revents;
         };

         typedef std::vector<PollFD> ListType;
         void add(Socket *sock, int events);
         void remove(const Socket& sock);
         void clear();

         ListType operator()(int64_t msec_timeout = 0);
      private:
         ListType m_list;
   };

   namespace Internal {
      struct remove_pred
      {
         remove_pred(const Socket& sock) : m_sock(sock) {}
         bool operator()(const Poller::PollFD& in) { return *(in.sock) == m_sock; }
         const Socket& m_sock;
      };
   }

}

#endif
