#include "Socket.hpp"
#include <sys/socket.h>
#include <netdb.h>
#include <netinet/in.h>
#include <netinet/tcp.h>
#include <unistd.h>
#include <sys/poll.h>
#include <string.h>
#include <arpa/inet.h>

#ifndef __CELLOS_LV2__
#include <fcntl.h>
#endif

namespace Network {

   Socket::Socket(int socktype) : m_type(socktype), m_alive(true)
   {
      m_fd = socket(AF_INET, socktype, 0);
      if (m_fd < 0)
         alive(false);
   }

   Socket::Socket(int socktype, int fd) : m_type(socktype), m_alive(true)
   {
      m_fd = fd;
      if (m_fd < 0)
         alive(false);
   }

   Socket::~Socket()
   {
#ifdef __CELLOS_LV2__
      socketclose(m_fd);
#else
      close(m_fd);
#endif
   }
   
   bool Socket::alive() const
   {
      return m_alive;
   }

   void Socket::alive(bool is_alive)
   {
      m_alive = is_alive;
   }

   bool Socket::is_blocking() const
   {
      return m_blocking;
   }

   void Socket::blocking(bool block)
   {
      if (!alive())
         return;

#ifdef __CELLOS_LV2__
      int i = !block;
      setsockopt(m_fd, SOL_SOCKET, SO_NBIO, &i, sizeof(i));
#else
      if (block)
         fcntl(m_fd, F_SETFL, fcntl(m_fd, F_GETFL) & (~O_NONBLOCK));
      else
         fcntl(m_fd, F_SETFL, fcntl(m_fd, F_GETFL) | O_NONBLOCK);
#endif
      m_blocking = block;
   }

   bool Socket::operator==(const Socket& in) const
   {
      return m_fd == in.m_fd && m_fd != -1;
   }

   int Socket::fd() const
   {
      return m_fd;
   }

   // Sockets need to implement this properly.
   ssize_t Socket::recv(void *, size_t)
   {
      return -1;
   }

   // Sockets need to implement this properly.
   ssize_t Socket::send(const void*, size_t)
   {
      return -1;
   }

   ssize_t Socket::send(const std::string& msg)
   {
      return send(msg.c_str(), msg.size());
   }

   ssize_t Socket::send(const char *msg)
   {
      return send(msg, strlen(msg));
   }
}

