#ifndef __NETWORK_SOCKET_HPP
#define __NETWORK_SOCKET_HPP

#include <unistd.h>
#include <string>
#ifdef __CELLOS_LV2__
#include <netex/net.h>
#include "network.hpp"
#endif

namespace Network {

#ifdef __CELLOS_LV2__
   class Socket : public NetworkInterface
#else
   class Socket
#endif
   {
      public:
         bool alive() const;
         bool is_blocking() const;
         void blocking(bool blocking = true);
         virtual ~Socket();

         virtual ssize_t recv(void *buf, size_t size);
         virtual ssize_t send(const void *buf, size_t size);
         ssize_t send(const std::string& msg);
         ssize_t send(const char *msg);
         bool operator==(const Socket&) const;
         int fd() const;

      protected:
         Socket(int socktype);
         Socket(int socktype, int fd);
         void alive(bool is_alive);

      private:
         void operator=(const Socket&);
         int m_fd;
         int m_type;
         bool m_alive;
         bool m_blocking;
   };

}

#endif
