#include "profile.h"
#include "../network/TCPSocket.hpp"

#include <map>
#include <string>
#include <sstream>
#include <utility>
#include <iostream>
#include <fstream>

using namespace std;

// Platform dependant
#ifdef __CELLOS_LV2__
#include <sys/sys_time.h>
static uint64_t get_usec()
{
   return sys_time_get_system_time();
}
#else
#include <time.h>
static uint64_t get_usec()
{
   struct timespec tv;
   clock_gettime(CLOCK_MONOTONIC, &tv);
   return (uint64_t)tv.tv_sec * 1000000 + (uint64_t)tv.tv_nsec / 1000;
}
#endif

class TimeInfo
{
   public:
      void start()
      {
         m_start = get_usec();
      }

      void stop()
      {
         m_stop = get_usec();
         m_delta = m_stop - m_start;
         m_total += m_delta;
         m_cnt++;
      }

      uint64_t clock() const
      {
         return m_delta;
      }

      uint64_t total_clock() const
      {
         return m_total;
      }

      uint64_t count() const
      {
         return m_cnt;
      }

      uint64_t avg_usec() const
      {
         return m_total / m_cnt;
      }

   private:
      uint64_t m_start, m_stop, m_cnt, m_delta, m_total;
};

static map<string, TimeInfo> g_map;

void perf_start(const char *ident)
{
   g_map[ident].start();
}

void perf_stop(const char *ident)
{
   g_map[ident].stop();
}

uint64_t perf_last_time(const char *ident)
{
   return g_map[ident].clock();
}

uint64_t perf_total_time(const char *ident)
{
   return g_map[ident].total_clock();
}

uint64_t perf_avg_time(const char *ident)
{
   return g_map[ident].avg_usec();
}

uint64_t perf_count(const char *ident)
{
   return g_map[ident].count();
}

void perf_dump_file(const char *path)
{
   fstream out(path, ios::out);
   if (!out.is_open())
      return;

   for (map<string, TimeInfo>::iterator itr = g_map.begin(); itr != g_map.end(); ++itr)
   {
      out << ":: " << itr->first << " ::" << endl << "\t" << "Calls: " << itr->second.count() << "   Avg call time (usec): " << itr->second.avg_usec() << "  Total call time (usec): " << itr->second.total_clock() << endl;
   }
}

void perf_dump_tcp(const char *host, uint16_t port)
{
   Network::TCPSocket sock(host, port);
   if (!sock.alive())
      return;
   
   ostringstream stream;
   for (map<string, TimeInfo>::iterator itr = g_map.begin(); itr != g_map.end() && sock.alive(); ++itr)
   {
      stream << ":: " << itr->first << " ::" << endl << "\t" << "Calls: " << itr->second.count() << "   Avg call time (usec): " << itr->second.avg_usec() << "  Total call time (usec): " << itr->second.total_clock() << endl;
      sock.send(stream.str());
      stream.str("");
   }
}

